@@ -8,7 +8,7 @@ use Carp;
 use vars qw( $SOCKET_CLASS @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION $SOCKS_ERROR $SOCKS5_RESOLVE $SOCKS4_RESOLVE $SOCKS_DEBUG %CODES );
 require Exporter;
 
-$VERSION = '0.67';
+$VERSION = '0.68';
 
 use constant {
 	SOCKS_WANT_READ  => 20,
@@ -143,7 +143,7 @@ use constant {
 	Q_DEBUGS => 6,
 };
 
-
+our $CAN_CHANGE_SOCKET = 1;
 sub new_from_fd {
 	my ($class, $sock, %arg) = @_;
 
@@ -155,6 +155,9 @@ sub new_from_fd {
 	}
 
 	scalar(%arg) or return $sock;
+	
+	# do not allow to create new socket
+	local $CAN_CHANGE_SOCKET = 0;
 	return $sock->configure(\%arg);
 }
 
@@ -178,13 +181,7 @@ sub start_SOCKS {
 sub socket {
 	my $self = shift;
 
-	if (-S $self) {
-		${*$self}{'io_socket_domain'} ||= $_[0];
-		${*$self}{'io_socket_type'}   ||= $_[1];
-		${*$self}{'io_socket_proto'}  ||= $_[2];
-		return $self;
-	}
-
+	return $self unless $CAN_CHANGE_SOCKET;
 	return $self->SUPER::socket(@_);
 }
 
@@ -366,6 +363,10 @@ sub connect {
 		$SOCKS_ERROR->set($!, $@ = "Connection to proxy failed: $!");
 		return;
 	}
+	else {
+		# connect() may be called several times by SUPER class
+		$SOCKS_ERROR->set();
+	}
 
 	$self->_connect();
 }
